@@ -2,7 +2,7 @@ package Catmandu::CQLSearchable;
 
 use Catmandu::Sane;
 
-our $VERSION = '1.10';
+our $VERSION = '1.2016';
 
 use Moo::Role;
 use namespace::clean;
@@ -53,7 +53,7 @@ Catmandu::CQLSearchable - Optional role for CQL searchable stores
 
     my $hits  = $store->bag->search(
            cql_query => 'keyword any dna',
-           sru_sortkeys  => 'title',
+           sru_sortkeys  => 'title,,0',
            limit => 100,
     );
 
