@@ -2,24 +2,22 @@ package Catmandu::Fix::from_json;
 
 use Catmandu::Sane;
 
-our $VERSION = '1.10';
+our $VERSION = '1.2016';
 
 use Cpanel::JSON::XS ();
 use Moo;
+use Catmandu::Util::Path qw(as_path);
 use namespace::clean;
 use Catmandu::Fix::Has;
 
+with 'Catmandu::Fix::Builder';
+
 has path => (fix_arg => 1);
 
-with 'Catmandu::Fix::SimpleGetValue';
-
-sub emit_value {
-    my ($self, $var, $fixer) = @_;
-    my $json_var = $fixer->capture(
-        Cpanel::JSON::XS->new->utf8(0)->pretty(0)->allow_nonref(1));
-
-    "if (is_string(${var})) {"
-        . "${var} = ${json_var}->decode(${var});" . "}";
+sub _build_fixer {
+    my ($self) = @_;
+    my $json = Cpanel::JSON::XS->new->utf8(0)->pretty(0)->allow_nonref(1);
+    as_path($self->path)->updater(if_string => sub {$json->decode($_[0])});
 }
 
 1;
