@@ -2,32 +2,22 @@ package Catmandu::Fix::set_array;
 
 use Catmandu::Sane;
 
-our $VERSION = '1.10';
+our $VERSION = '1.2016';
 
 use Moo;
+use Catmandu::Util::Path qw(as_path);
 use namespace::clean;
 use Catmandu::Fix::Has;
 
-with 'Catmandu::Fix::Base';
+with 'Catmandu::Fix::Builder';
 
-has path => (fix_arg => 1);
+has path   => (fix_arg => 1);
 has values => (fix_arg => 'collect', default => sub {[]});
 
-sub emit {
-    my ($self, $fixer) = @_;
-    my $path   = $fixer->split_path($self->path);
-    my $key    = pop @$path;
+sub _build_fixer {
+    my ($self) = @_;
     my $values = $self->values;
-
-    $fixer->emit_walk_path(
-        $fixer->var,
-        $path,
-        sub {
-            my $var = shift;
-            $fixer->emit_set_key($var, $key,
-                "[" . join(',', map {$fixer->emit_value($_)} @$values) . "]");
-        }
-    );
+    as_path($self->path)->setter(sub {[@$values]});
 }
 
 1;
