@@ -2,9 +2,18 @@ package Catmandu::Iterable;
 
 use Catmandu::Sane;
 
-our $VERSION = '0.9505';
+our $VERSION = '1.2016';
 
-use Catmandu::Util qw(:is :check);
+use Catmandu::Util qw(
+    is_number
+    is_value
+    is_string
+    is_array_ref
+    is_hash_ref
+    is_regex_ref
+    is_same
+    check_positive
+);
 use Time::HiRes qw(gettimeofday tv_interval);
 use Hash::Util::FieldHash qw(fieldhash);
 use Role::Tiny;
@@ -44,8 +53,8 @@ sub to_array {
 
 sub count {
     my ($self) = @_;
-    my $next = $self->generator;
-    my $n = 0;
+    my $next   = $self->generator;
+    my $n      = 0;
     while ($next->()) {
         $n++;
     }
@@ -55,30 +64,34 @@ sub count {
 sub slice {
     my ($self, $start, $total) = @_;
     $start //= 0;
-    Catmandu::Iterator->new(sub { sub {
-        if (defined $total) {
-            $total || return;
-        }
-        state $next = $self->generator;
-        state $data;
-        while (defined($data = $next->())) {
-            if ($start > 0) {
-                $start--;
-                next;
+    Catmandu::Iterator->new(
+        sub {
+            sub {
+                if (defined $total) {
+                    $total || return;
+                }
+                state $next = $self->generator;
+                state $data;
+                while (defined($data = $next->())) {
+                    if ($start > 0) {
+                        $start--;
+                        next;
+                    }
+                    if (defined $total) {
+                        $total--;
+                    }
+                    return $data;
+                }
+                return;
             }
-            if (defined $total) {
-                $total--;
-            }
-            return $data;
         }
-        return;
-    }});
+    );
 }
 
 sub each {
     my ($self, $sub) = @_;
     my $next = $self->generator;
-    my $n = 0;
+    my $n    = 0;
     my $data;
     while (defined($data = $next->())) {
         $sub->($data);
@@ -90,7 +103,7 @@ sub each {
 sub each_until {
     my ($self, $sub) = @_;
     my $next = $self->generator;
-    my $n = 0;
+    my $n    = 0;
     my $data;
     while (defined($data = $next->())) {
         $sub->($data) || last;
@@ -101,15 +114,41 @@ sub each_until {
 
 sub tap {
     my ($self, $sub) = @_;
-    Catmandu::Iterator->new(sub { sub {
-        state $next = $self->generator;
-        state $data;
-        if (defined($data = $next->())) {
-            $sub->($data);
-            return $data;
+    Catmandu::Iterator->new(
+        sub {
+            sub {
+                state $next = $self->generator;
+                state $data;
+                if (defined($data = $next->())) {
+                    $sub->($data);
+                    return $data;
+                }
+                return;
+            }
         }
-        return;
-    }});
+    );
+}
+
+sub every {
+    my ($self, $n, $sub) = @_;
+    check_positive($n);
+    Catmandu::Iterator->new(
+        sub {
+            sub {
+                state $next = $self->generator;
+                state $data;
+                state $i = 0;
+                if (defined($data = $next->())) {
+                    if (++$i == $n) {
+                        $sub->($data);
+                        $i = 0;
+                    }
+                    return $data;
+                }
+                return;
+            }
+        }
+    );
 }
 
 sub any {
@@ -125,7 +164,7 @@ sub any {
 sub many {
     my ($self, $sub) = @_;
     my $next = $self->generator;
-    my $n = 0;
+    my $n    = 0;
     my $data;
     while (defined($data = $next->())) {
         $sub->($data) && ++$n > 1 && return 1;
@@ -145,24 +184,31 @@ sub all {
 
 sub map {
     my ($self, $sub) = @_;
-    Catmandu::Iterator->new(sub { sub {
-        state $next = $self->generator;
-        $sub->($next->() // return);
-    }});
+    Catmandu::Iterator->new(
+        sub {
+            sub {
+                state $next = $self->generator;
+                state @buff;
+                @buff = $sub->($next->() // return) unless @buff;
+                shift @buff;
+            }
+        }
+    );
 }
 
 sub reduce {
-    my $self = shift;
+    my $self     = shift;
     my $memo_set = @_ > 1;
-    my $sub  = pop;
-    my $memo = shift;
-    my $next = $self->generator;
+    my $sub      = pop;
+    my $memo     = shift;
+    my $next     = $self->generator;
     my $data;
     while (defined($data = $next->())) {
         if ($memo_set) {
             $memo = $sub->($memo, $data);
-        } else {
-            $memo = $data;
+        }
+        else {
+            $memo     = $data;
             $memo_set = 1;
         }
     }
@@ -189,12 +235,13 @@ sub take {
             if (is_regex_ref($arg2)) {
                 return sub {
                     is_hash_ref($_[0]) || return 0;
-                    my $val = $_[0]->{$arg1}; is_value($val) && $val =~ $arg2;
+                    my $val = $_[0]->{$arg1};
+                    is_value($val) && $val =~ $arg2;
                 };
             }
             if (is_array_ref($arg2)) {
                 return sub {
-                    is_hash_ref($_[0]) || return 0;
+                    is_hash_ref($_[0])                 || return 0;
                     is_value(my $val = $_[0]->{$arg1}) || return 0;
                     for my $v (@$arg2) {
                         return 1 if $val eq $v;
@@ -204,13 +251,15 @@ sub take {
             }
             return sub {
                 is_hash_ref($_[0]) || return 0;
-                my $val = $_[0]->{$arg1}; is_value($val) && $val eq $arg2;
+                my $val = $_[0]->{$arg1};
+                is_value($val) && $val eq $arg2;
             };
         }
 
         if (is_regex_ref($arg1)) {
             return sub {
-                my $val = $_[0]; is_value($val) && $val =~ $arg1;
+                my $val = $_[0];
+                is_value($val) && $val =~ $arg1;
             };
         }
 
@@ -218,7 +267,8 @@ sub take {
     };
 
     sub detect {
-        my $self = shift; my $sub = $to_sub->(@_);
+        my $self = shift;
+        my $sub  = $to_sub->(@_);
         my $next = $self->generator;
         my $data;
         while (defined($data = $next->())) {
@@ -228,174 +278,212 @@ sub take {
     }
 
     sub select {
-        my $self = shift; my $sub = $to_sub->(@_);
-        Catmandu::Iterator->new(sub { sub {
-            state $next = $self->generator;
-            state $data;
-            while (defined($data = $next->())) {
-                $sub->($data) && return $data;
+        my $self = shift;
+        my $sub  = $to_sub->(@_);
+        Catmandu::Iterator->new(
+            sub {
+                sub {
+                    state $next = $self->generator;
+                    state $data;
+                    while (defined($data = $next->())) {
+                        $sub->($data) && return $data;
+                    }
+                    return;
+                }
             }
-            return;
-        }});
+        );
     }
 
-    sub grep { goto &select }
-    
+    sub grep {goto &select}
+
     sub reject {
-        my $self = shift; my $sub = $to_sub->(@_);
-        Catmandu::Iterator->new(sub { sub {
-            state $next = $self->generator;
-            state $data;
-            while (defined($data = $next->())) {
-                $sub->($data) || return $data;
+        my $self = shift;
+        my $sub  = $to_sub->(@_);
+        Catmandu::Iterator->new(
+            sub {
+                sub {
+                    state $next = $self->generator;
+                    state $data;
+                    while (defined($data = $next->())) {
+                        $sub->($data) || return $data;
+                    }
+                    return;
+                }
             }
-            return;
-        }});
+        );
     }
 };
 
 sub sorted {
     my ($self, $cmp) = @_;
     if (!defined $cmp) {
-        Catmandu::ArrayIterator->new([ sort @{$self->to_array} ]);
-    } elsif (ref $cmp) {
-        Catmandu::ArrayIterator->new([ 
-            sort { $cmp->($a,$b) }
-            @{$self->to_array} 
-        ]);
-    } else {
+        Catmandu::ArrayIterator->new([sort @{$self->to_array}]);
+    }
+    elsif (ref $cmp) {
+        Catmandu::ArrayIterator->new(
+            [sort {$cmp->($a, $b)} @{$self->to_array}]);
+    }
+    else {
         # TODO: use Schwartzian transform for more complex key
-        Catmandu::ArrayIterator->new([ 
-            sort { $a->{$cmp} cmp $b->{$cmp} } @{$self->to_array}
-        ]);
+        Catmandu::ArrayIterator->new(
+            [sort {$a->{$cmp} cmp $b->{$cmp}} @{$self->to_array}]);
     }
 }
 
 sub pluck {
     my ($self, $key) = @_;
-    $self->map(sub {
-        $_[0]->{$key};
-    });
+    $self->map(
+        sub {
+            $_[0]->{$key};
+        }
+    );
 }
 
 sub invoke {
     my ($self, $method, @args) = @_;
-    $self->map(sub {
-        $_[0]->$method(@args);
-    });
+    $self->map(
+        sub {
+            $_[0]->$method(@args);
+        }
+    );
 }
 
-sub contains { goto &includes }
+sub contains {goto &includes}
 
 sub includes {
     my ($self, $data) = @_;
-    $self->any(sub {
-        is_same($data, $_[0]);
-    });
+    $self->any(
+        sub {
+            is_same($data, $_[0]);
+        }
+    );
 }
 
 sub group {
     my ($self, $size) = @_;
-    Catmandu::Iterator->new(sub { sub {
-        state $next = $self->generator;
+    Catmandu::Iterator->new(
+        sub {
+            sub {
+                state $next = $self->generator;
 
-        my $group = [];
+                my $group = [];
 
-        for (my $i = 0; $i < $size; $i++) {
-            push @$group, $next->() // last;
+                for (my $i = 0; $i < $size; $i++) {
+                    push @$group, $next->() // last;
+                }
+
+                unless (@$group) {
+                    return;
+                }
+
+                Catmandu::ArrayIterator->new($group);
+            }
         }
-
-        unless (@$group) {
-            return;
-        }
-
-        Catmandu::ArrayIterator->new($group);
-    }});
+    );
 }
 
 sub interleave {
     my @iterators = @_;
-    Catmandu::Iterator->new(sub { sub {
-        state @generators;
-        state $n = @iterators;
-        state $i = 0;
-        while ($n) {
-            $i = 0 if $i == $n;
-            my $next = $generators[$i] ||= $iterators[$i]->generator;
-            if (defined(my $data = $next->())) {
-                $i++;
-                return $data;
-            } else {
-                splice @generators, $i, 1;
-                $n--;
+    Catmandu::Iterator->new(
+        sub {
+            sub {
+                state @generators;
+                state $n = @iterators;
+                state $i = 0;
+                while ($n) {
+                    $i = 0 if $i == $n;
+                    my $next = $generators[$i] ||= $iterators[$i]->generator;
+                    if (defined(my $data = $next->())) {
+                        $i++;
+                        return $data;
+                    }
+                    else {
+                        splice @generators, $i, 1;
+                        $n--;
+                    }
+                }
+                return;
             }
         }
-        return;
-    }});
+    );
 }
 
 sub max {
     my ($self, $sub) = @_;
-    $self->reduce(undef, sub {
-        my ($memo, $data) = @_;
-        my $val = defined $sub ? $sub->($data) : $data;
-        return $val > $memo ? $val : $memo if is_number($memo) && is_number($val);
-        return $memo if is_number($memo);
-        return $val  if is_number($val);
-        return;
-    });
+    $self->reduce(
+        undef,
+        sub {
+            my ($memo, $data) = @_;
+            my $val = defined $sub ? $sub->($data) : $data;
+            return $val > $memo ? $val : $memo
+                if is_number($memo) && is_number($val);
+            return $memo if is_number($memo);
+            return $val  if is_number($val);
+            return;
+        }
+    );
 }
 
 sub min {
     my ($self, $sub) = @_;
-    $_[0]->reduce(undef, sub {
-        my ($memo, $data) = @_;
-        my $val = defined $sub ? $sub->($data) : $data;
-        return $val < $memo ? $val : $memo if is_number($memo) && is_number($val);
-        return $memo if is_number($memo);
-        return $val  if is_number($val);
-        return;
-    });
+    $_[0]->reduce(
+        undef,
+        sub {
+            my ($memo, $data) = @_;
+            my $val = defined $sub ? $sub->($data) : $data;
+            return $val < $memo ? $val : $memo
+                if is_number($memo) && is_number($val);
+            return $memo if is_number($memo);
+            return $val  if is_number($val);
+            return;
+        }
+    );
 }
 
 sub benchmark {
     my ($self) = @_;
-    $self->tap(sub {
-        state $n = 0;
-        state $t = [gettimeofday];
-        if (++$n % 100 == 0) {
-            printf STDERR "added %9d (%d/sec)\n", $n, $n/tv_interval($t);
+    $self->tap(
+        sub {
+            state $n = 0;
+            state $t = [gettimeofday];
+            if (++$n % 100 == 0) {
+                printf STDERR "added %9d (%d/sec)\n", $n,
+                    $n / tv_interval($t);
+            }
         }
-    });
+    );
 }
 
 sub format {
     my ($self, %opts) = @_;
-    $opts{header} //= 1;
+    $opts{header}  //= 1;
     $opts{col_sep} //= " | ";
-    my @cols = $opts{cols} ? @{$opts{cols}} : ();
+    my @cols        = $opts{cols} ? @{$opts{cols}} : ();
     my @col_lengths = map length, @cols;
 
-    my $rows = $self->map(sub {
-        my $data = $_[0];
-        my $row = [];
-        for (my $i = 0; $i < @cols; $i++) {
-            my $col = $data->{$cols[$i]} // "";
-            my $len = length $col;
-            $col_lengths[$i] = $len if $len > $col_lengths[$i];
-            push @$row, $col;
+    my $rows = $self->map(
+        sub {
+            my $data = $_[0];
+            my $row  = [];
+            for (my $i = 0; $i < @cols; $i++) {
+                my $col = $data->{$cols[$i]} // "";
+                my $len = length $col;
+                $col_lengths[$i] = $len if $len > $col_lengths[$i];
+                push @$row, $col;
+            }
+            $row;
         }
-        $row;
-    })->to_array;
+    )->to_array;
 
-    my @indices = 0 .. @cols-1;
-    my $pattern = join($opts{col_sep}, map { "%-$col_lengths[$_]s" } @indices)."\n";
+    my @indices = 0 .. @cols - 1;
+    my $pattern
+        = join($opts{col_sep}, map {"%-$col_lengths[$_]s"} @indices) . "\n";
 
     if ($opts{header}) {
         printf $pattern, @cols;
         my $sep = $opts{col_sep};
         $sep =~ s/[^|]/-/g;
-        print join($sep, map { '-' x $col_lengths[$_] } @indices)."\n";
+        print join($sep, map {'-' x $col_lengths[$_]} @indices) . "\n";
     }
     for my $row (@$rows) {
         printf $pattern, @$row;
@@ -404,12 +492,26 @@ sub format {
 
 sub stop_if {
     my ($self, $sub) = @_;
-    Catmandu::Iterator->new(sub { sub {
-        state $next = $self->generator;
-        my $data = $next->() // return;
-        $sub->($data) && return;
-        $data;
-    }});
+    Catmandu::Iterator->new(
+        sub {
+            sub {
+                state $next = $self->generator;
+                my $data = $next->() // return;
+                $sub->($data) && return;
+                $data;
+            }
+        }
+    );
+}
+
+sub run {
+    my ($self) = @_;
+    my $next   = $self->generator;
+    my $run    = 0;
+    while (defined($next->())) {
+        $run ||= 1;
+    }
+    $run;
 }
 
 1;
@@ -420,7 +522,7 @@ __END__
 
 =head1 NAME
 
-Catmandu::Iterable - Base class for all iterable Catmandu classes
+Catmandu::Iterable - Base role for all iterable Catmandu classes
 
 =head1 SYNOPSIS
 
@@ -505,42 +607,54 @@ a second invocation.
 
 =head2 to_array
 
-Return all the items in the Iterator as an ARRAY ref.
+Return all the items in the iterator as an array ref.
 
 =head2 count
 
-Return the count of all the items in the Iterator.
+Return the count of all the items in the iterator.
 
 =head3 LOOPING
 
 =head2 each(\&callback)
 
-For each item in the Iterator execute the callback function with the item as first argument. Returns
-the number of items in the Iterator.
+For each item in the iterator execute the callback function with the item as
+first argument. Returns the number of items in the iterator.
 
 =head2 first
 
-Return the first item from the Iterator.
+Return the first item from the iterator.
 
 =head2 rest
 
-Returns an Iterator containing everything except the first item.
+Returns an iterator containing everything except the first item.
 
-=head2 slice(INDEX,LENGTH)
+=head2 slice($index,$length)
 
-Returns an Iterator starting at the item at INDEX returning at most LENGTH results.
+Returns an new iterator starting at the item at C<$index> returning at most <$length> items.
 
-=head2 take(NUM)
+=head2 take($num)
 
-Returns an Iterator with the first NUM results.
+Returns an iterator with the first C<$num> items.
 
-=head2 group(NUM)
+=head2 group($num)
 
-Splitting the Iterator into NUM parts and returning an Iterator for each part.
+Splits the iterator into new iterators each containing C<$num> items.
+
+    $it->group(500)->each(sub {
+        my $group_it = $_[0];
+        $group_it->each(sub {
+            my $item = $_[0];
+            # ...
+        });
+    });
+
+Note that the group iterators load their items in memory. The last group
+iterator will contain less than C<$num> item unless the item count is divisible
+by C<$num>.
 
 =head2 interleave(@iterators)
 
-Returns an Iterator which returns the first item of each iterator then the
+Returns an iterator which returns the first item of each iterator then the
 second of each and so on.
 
 =head2 contains($data)
@@ -553,8 +667,8 @@ return true if any item in the collection is deeply equal to C<$data>.
 
 =head2 tap(\&callback)
 
-Returns a copy of the Iterator and executing callback on each item. This method works
-like the Unix L<tee> command. Use this command to peek into an iterable while it is
+Returns a copy of the iterator and executing callback on each item. This method works
+like the Unix C<tee> command. Use this command to peek into an iterable while it is
 processing results. E.g. you are writing code to process an iterable and wrote
 something like:
 
@@ -583,6 +697,11 @@ execution time.
 
 Note that the C<benchmark> method already implements this common case.
 
+=head2 every($num, \&callback)
+
+Similar to C<tap>, but only calls the callback every C<$num> times. Useful for
+benchmarking and sampling.
+
 =head2 detect(\&callback)
 
 Returns the first item for which callback returns a true value.
@@ -595,25 +714,25 @@ regex.
 =head2 detect($key => $val)
 
 If the iterator contains HASH values, then return the first item where the value of
-$key is equal to val.
+C<$key> is equal to C<$val>.
 
 =head2 detect($key => qr/..../)
 
 If the iterator contains HASH values, then return the first item where the value of
-$key matches the regex.
+C<$key> matches the regex.
 
 =head2 detect($key => [$val, ...])
 
 If the iterator contains HASH values, then return the first item where the value of
-$key is equal to any of the vals given.
+C<$key> is equal to any of the values given.
 
 =head2 pluck($key)
 
-Return an Iterator that only containes the values of the given $key.
+Return an iterator that only contains the values of the given C<$key>.
 
 =head2 select(\&callback)
 
-Returns an Iterator for each item for which callback returns a true value.
+Returns an iterator containing only items item for which the callback returns a true value.
 
 =head2 select(qr/..../)
 
@@ -622,17 +741,17 @@ If the iterator contains STRING values, then return each item which matches the
 =head2 select($key => $val)
 
 If the iterator contains HASH values, then return each item where the value of
-$key is equal to val.
+C<$key> is equal to C<$val>.
 
 =head2 select($key => qr/..../)
 
-If the iterator contains HASH values, then return each item where the value of $key
+If the iterator contains HASH values, then return each item where the value of C<$key>
 matches the regex.
 
 =head2 select($key => [$val, ...])
 
 If the iterator contains HASH values, then return each item where the value of
-$key is equal to any of the vals given.
+C<$key> is equal to any of the vals given.
 
 =head2 grep( ... )
 
@@ -640,7 +759,7 @@ Alias for C<select( ... )>.
 
 =head2 reject(\&callback)
 
-Returns an Iterator for each item for which callback returns a false value.
+Returns an iterator containing each item for which callback returns a false value.
 
 =head2 reject(qr/..../)
 
@@ -649,36 +768,36 @@ matching the regex.
 
 =head2 reject($key => qr/..../)
 
-If the iterator contains HASH values, then reject every item for where the value of $key
+If the iterator contains HASH values, then reject every item for where the value of C<$key>
 DOESN'T match the regex.
 
 =head2 reject($key => $val)
 
 If the iterator contains HASH values, then return each item where the value of
-$key is NOT equal to val.
+C<$key> is NOT equal to C<$val>.
 
 =head2 reject($key => [$val, ...])
 
 If the iterator contains HASH values, then return each item where the value of
-$key is NOT equal to any of the vals given.
+C<$key> is NOT equal to any of the values given.
 
 =head2 sorted
 
-Returns an Iterator with items sorted lexically. Note that sorting requires
+Returns an iterator with items sorted lexically. Note that sorting requires
 memory because all items are buffered in a L<Catmandu::ArrayIterator>.
 
 =head2 sorted(\&callback)
 
-Returns an Iterator with items sorted by a callback. The callback is expected to
+Returns an iterator with items sorted by a callback. The callback is expected to
 returns an integer less than, equal to, or greater than C<0>. The following code
-snippets result in the equal arrays:
+snippets result in equal arrays:
 
     $iterator->sorted(\&callback)->to_array
     [ sort \&callback @{ $iterator->to_array } ] 
 
 =head2 sorted($key) 
 
-Returns an Iterator with items lexically sorted by a key. This is equivalent to
+Returns an iterator with items lexically sorted by a key. This is equivalent to
 sorting with the following callback:
 
     $iterator->sorted(sub { $_[0]->{$key} cmp $_[1]->{$key} })
@@ -734,22 +853,20 @@ Returns true if all the items generate a true value when executing callback.
 
 =head2 map(\&callback)
 
-Returns a new Iterator containing for each item the result of the callback.
+Returns a new iterator containing for each item the result of the callback. If
+the callback returns multiple or no items, the resulting iterator will grow or
+shrink.
 
-=head2 reduce([START],\&callback)
+=head2 reduce([$start],\&callback)
 
-Alias for C<reduce>.
-
-=head2 reduce([START],\&callback)
-
-For each item in the Iterator execute &callback($prev,$item) where $prev is the
-option START value or the result of the previous call to callback. Returns the
+For each item in the iterator execute C<&callback($prev,$item)> where C<$prev> is the
+optional C<$start> value or the result of the previous call to callback. Returns the
 final result of the callback function.
 
-=head2 invoke(NAME)
+=head2 invoke($name)
 
-Returns an interator were the method NAME is called on every object in the iterable.
-This is a shortcut for $it->map(sub { $_[0]->NAME }).
+Returns an interator were the method C<$name> is called on every object in the iterable.
+This is a shortcut for C<$it->map(sub { $_[0]->$name })>.
 
 =head2 max()
 
@@ -793,6 +910,20 @@ Returns a new iterator thats stops processing if the callback returns false.
         ...
     });
 
+=head2 run
+
+Simply invokes the iterator and returns 1 if any records were processed, 0 otherwise.
+
+    $it = $it->tap(sub {
+        # do something
+    });
+    $it = $it->tap(sub {
+        # do another thing
+    });
+    $it->run
+
+    print 'not empty' if $it->run;
+
 =head1 SEE ALSO
 
 L<Catmandu::Iterator>.
