@@ -2,7 +2,7 @@ package Catmandu::Paged;
 
 use Catmandu::Sane;
 
-our $VERSION = '1.2012';
+our $VERSION = '1.2016';
 
 use Moo::Role;
 use namespace::clean;
@@ -121,12 +121,18 @@ sub _do_pagination {
 }
 
 sub first_page {
+    my $self = shift;
+
+    return if $self->limit < 1;
+
     return 1;
 }
 
 sub last_page {
     my $self = shift;
 
+    return if $self->limit < 1;
+
     my $last = $self->_capped_total / $self->limit;
     return _ceil($last);
 }
@@ -134,6 +140,8 @@ sub last_page {
 sub page {
     my $self = shift;
 
+    return if $self->limit < 1;
+
     ($self->start == 0) && (return 1);
 
     my $page = _ceil(($self->start + 1) / $self->limit);
@@ -143,12 +151,16 @@ sub page {
 sub previous_page {
     my $self = shift;
 
+    return if $self->limit < 1;
+
     ($self->page > 1) ? (return $self->page - 1) : (return undef);
 }
 
 sub next_page {
     my $self = shift;
 
+    return if $self->limit < 1;
+
     ($self->page < $self->last_page)
         ? (return $self->page + 1)
         : (return undef);
@@ -157,27 +169,31 @@ sub next_page {
 sub first_on_page {
     my $self = shift;
 
-    ($self->total == 0)
-        ? (return 0)
-        : (return (($self->page - 1) * $self->limit) + 1);
+    return 0 if $self->limit < 1 || $self->total == 0;
+
+    return (($self->page - 1) * $self->limit) + 1;
 }
 
 sub last_on_page {
     my $self = shift;
 
+    return 0 if $self->limit < 1;
+
     ($self->page == $self->last_page)
         ? (return $self->_capped_total)
         : (return ($self->page * $self->limit));
 }
 
 sub page_size {
-    my $self = shift,;
+    my $self = shift;
     return $self->limit;
 }
 
 sub page_ranges {
     my $self = shift;
 
+    return if $self->limit < 1;
+
     $self->_do_pagination;
     return @{$self->{PAGE_RANGES}};
 }
@@ -185,6 +201,8 @@ sub page_ranges {
 sub pages_in_spread {
     my $self = shift;
 
+    return [] if $self->limit < 1;
+
     $self->_do_pagination;
     my $ranges = $self->{PAGE_RANGES};
     my $pages  = [];
